"use strict";
/*
 * This program and the accompanying materials are made available under the terms of the
 * Eclipse Public License v2.0 which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-v20.html
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Copyright Contributors to the Zowe Project.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@zowe/imperative");
const path = require("path");
class BaseScsHandler {
    /**
     * Update profiles to be secure, or revert them to plain text
     * @param params Command handler parameters
     * @param shouldBeSecure Specifies whether credentials should be secure
     * @returns False if secure credential manager is not enabled
     */
    static updateProfiles(params, shouldBeSecure) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.ensureCredMgrIsEnabled();
            }
            catch (err) {
                params.response.console.errorHeader("Error");
                params.response.console.error(err.message);
                params.response.data.setExitCode(1);
                return false;
            }
            // Boolean to check if user has profiles or not
            let noProfiles = true;
            // profiles folder
            const profilesRootDir = imperative_1.ProfileUtils.constructProfilesRootDirectory(imperative_1.ImperativeConfig.instance.cliHome);
            const listOfProfileTypes = imperative_1.ProfileIO.getAllProfileDirectories(profilesRootDir);
            for (const profileType of listOfProfileTypes) {
                // Generic profile configuration
                const profileConfig = { profileRootDirectory: profilesRootDir, type: profileType };
                // Base directory for the given type of profile
                const profileTypeDir = path.join(profilesRootDir, profileType);
                // Names of the profiles of this type
                const profileNames = yield imperative_1.ProfileIO.getAllProfileNames(profileTypeDir, ".yaml", `${profileType}_meta`);
                // Profile type meta file
                const profileMetaFile = imperative_1.ProfileIO.readMetaFile(path.join(profileTypeDir, `${profileType}_meta.yaml`));
                // Check to see if there are no profiles to be updated
                if (profileNames.length > 0) {
                    noProfiles = false;
                }
                for (const profileName of profileNames) {
                    const profileData = {
                        profile: imperative_1.ProfileIO.readProfileFile(path.join(profileTypeDir, `${profileName}.yaml`), profileType),
                        profileName, profileType, profileConfig, profileMetaFile, profileTypeDir
                    };
                    try {
                        let result;
                        if (shouldBeSecure) {
                            result = yield this.makeProfileSecure(profileData);
                        }
                        else {
                            result = yield this.makeProfileInsecure(profileData);
                        }
                        params.response.console.log(result);
                    }
                    catch (err) {
                        params.response.console.errorHeader("Error");
                        params.response.console.error(err.message);
                    }
                }
            }
            if (noProfiles) {
                params.response.console.log("There are no profiles to update");
            }
            return true;
        });
    }
    /**
     * Check that CredentialManager is set in Imperative settings. If it is not,
     * throw an error.
     * @throws {Error}
     */
    static ensureCredMgrIsEnabled() {
        var _a;
        const credMgrSetting = (_a = imperative_1.ImperativeConfig.instance.loadedConfig.overrides) === null || _a === void 0 ? void 0 : _a.CredentialManager;
        if (credMgrSetting == null) {
            const pluginName = require(path.join(__dirname, "../../package.json")).name;
            throw new Error(`Invalid credential manager setting\n\n` +
                "Possible Causes:\n" +
                `  Expected secure credential manager to be enabled\n\n` +
                "Resolutions:\n" +
                `  Run "zowe config set CredentialManager ${pluginName}"\n`);
        }
    }
    /**
     * Check if a profile is already secure. We assume it is secure, if it
     * contains a property whose value starts with the string "managed by" and
     * is marked as secure in the profile schema.
     * @param pd Profile data object
     * @returns True if the profile is already secure
     */
    static isProfileSecure(pd) {
        for (const profileProperty in pd.profile) {
            // Does this property start with the words 'managed by'?
            if (pd.profile[profileProperty].toString().indexOf(imperative_1.ProfilesConstants.PROFILES_OPTION_SECURELY_STORED) === 0) {
                // If so, Is this property secure?
                if (pd.profileMetaFile.configuration.schema.properties[profileProperty].secure) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * Revert a secure profile to be stored in plain text.
     * @param pd Profile data object
     * @returns String to be logged to the console
     */
    static makeProfileInsecure(pd) {
        return __awaiter(this, void 0, void 0, function* () {
            // Indicator of whether a profile is already secured
            const alreadySecured = this.isProfileSecure(pd);
            if (!alreadySecured) {
                // Do not overwrite a profile seems to be already plain text
                return `Profile ("${pd.profileName} of type "${pd.profileType}") was already plain text`;
            }
            else {
                // Load profile with secure fields
                const profileLoaded = yield new imperative_1.CliProfileManager(pd.profileConfig).load({
                    name: pd.profileName,
                    type: pd.profileType
                });
                // Simple save profile with plain text fields
                const profileFilePath = path.join(pd.profileTypeDir, `${pd.profileName}.yaml`);
                imperative_1.ProfileIO.writeProfile(profileFilePath, profileLoaded.profile);
                // Clean up secure fields
                for (const profileProperty in pd.profile) {
                    // Does this property start with the words 'managed by'?
                    if (pd.profile[profileProperty].toString().indexOf(imperative_1.ProfilesConstants.PROFILES_OPTION_SECURELY_STORED) === 0) {
                        // If so, Is this property secure?
                        if (pd.profileMetaFile.configuration.schema.properties[profileProperty].secure) {
                            yield imperative_1.CredentialManagerFactory.manager.delete(imperative_1.ProfileUtils.getProfilePropertyKey(pd.profileType, pd.profileName, profileProperty));
                        }
                    }
                }
                return `Profile ("${pd.profileName}" of type "${pd.profileType}") successfully written: ${profileFilePath}`;
            }
        });
    }
    /**
     * Update a plain text profile to be stored securely.
     * @param pd Profile data object
     * @returns String to be logged to the console
     */
    static makeProfileSecure(pd) {
        return __awaiter(this, void 0, void 0, function* () {
            // Indicator of whether a profile is already secured
            const alreadySecured = this.isProfileSecure(pd);
            if (alreadySecured) {
                // Do not overwrite a profile seems to be already secure
                return `Profile ("${pd.profileName} of type "${pd.profileType}") was already secure`;
            }
            else {
                // Overwrite this plain text profiles
                const res = yield new imperative_1.CliProfileManager(pd.profileConfig).save({
                    name: pd.profileName,
                    type: pd.profileType,
                    profile: pd.profile,
                    updateDefault: false,
                    overwrite: true
                });
                return res.message;
            }
        });
    }
}
exports.default = BaseScsHandler;
//# sourceMappingURL=scs.shared.handler.js.map